--------------------------------------------------------------------------------
-- Copyright  2019								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- For more information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local AddonName, NS = ...

local L
local Container, ContextMenu
local FrameWidth, FrameHeight = 130

local ceil = math.ceil
local tinsert, tremove = table.insert, table.remove
local upper = string.upper

local ContextTypes, DropZone, Left, Overlay, Right, SaveDrag, Top, Width
local OptionsTable = {}

local function IsKnown(oftype, info)
	local value 
	if oftype == "spell" then
		value = GetSpellInfo(info)
		if value == nil then return end
		value = GetSpellInfo(value)
		return value
	elseif oftype == "item" then
		for i = 0, NUM_BAG_SLOTS do
			local numslots = GetContainerNumSlots and GetContainerNumSlots(i) or C_Container.GetContainerNumSlots(i)
			for j = 1, numslots do 
				local item = GetContainerItemID and GetContainerItemID(i, j) or C_Container.GetContainerItemID(i, j)
				if item == info then
					return true			
				end
			end
		end
		return ToyBox_FindPageForToyID and ToyBox_FindPageForToyID(info) or nil
	elseif oftype == "mount" then
		return C_MountJournal.GetMountInfoByID(info)
	end
end

local function CopyMenu(source, target)
	for key, value in pairs(source) do
		if type(value) == "table" then
			if not value.oftype then
				target[key] = {}
				CopyMenu(value, target[key])
			else
				if IsKnown(value.oftype, value.info) then
					local k = #target + 1
					target[k] = {}
					CopyMenu(value, target[k])
				end
			end
		else
			target[key] = value
		end		
	end
end

local function ConfigureMenu(parent)
	if not CollectionsJournal then
		local Loadable, Availability = select(4, GetAddOnInfo("Blizzard_Collections"))
		if Loadable and Availability ~= "MISSING" then
			CollectionsJournal_LoadUI()
		end
	end
	L = NS.TEXTS
	if Container then
		Container:SetShown(not Container:IsShown())
		return Container
	end
	FrameHeight = parent.Menu.MinHeight
	local CheckButtons = {}
	local function CopySettings(character, charsettings)
		local SrcSettings = NS:CharacterSettings(set, charsettings, "Menu")
		local Settings = NS:GetSettings("Menu")
		for k, set in pairs(CheckButtons) do
			k:Init(SrcSettings[k.Index] or false)
			k:GetScript("OnClick")(k)
		end
		table.wipe(Settings.pages)
		CopyMenu(SrcSettings.pages, Settings.pages)

		OptionsTable.LoadContextMenu()
	end
	local function GetCol(x)
		local col = ceil((x-Left)/Width)
		if col < 1 then
			col = 1
		end
		if col > 3 then
			col = 3
		end
		return col
	end

	local function OnEnterLeave(self, motion, leave)
		if leave then 
			DropZone:SetPos()
			self:SetScript("OnUpdate", nil)
		else
			if not GetCursorInfo() or GetMouseFocus() ~= Overlay then return end
			self.Column = 0
			self:SetScript("OnUpdate", function(self) 
				local x, y = GetCursorPosition()
				if  x > Left and y < Top and self.Column ~= GetCol(x) then
					self.Column = GetCol(x)
					DropZone:SetPos(self.Column)
				end
			end)
		end
	end

	local function TakeEvent(self, event, ...)
--		if WOW_PROJECT_ID == WOW_PROJECT_CLASSIC then
		if WOW_PROJECT_ID == WOW_PROJECT_MAINLINE then
			Overlay:UnregisterEvent("CURSOR_CHANGED")
		else
			Overlay:UnregisterEvent("CURSOR_UPDATE")
		end
		if Overlay:IsMouseOver() then  -- GetMouseFocus() maybe???
			SaveDrag = nil
			return 
		end
		local settings = NS:GetSettings("Menu").pages
		tremove(settings[SaveDrag.FMContextPage], SaveDrag.FMContextId)
		OptionsTable.LoadContextMenu()
		DropZone:SetPos(1, true)
	end

	local function TakeDrag(self)
		if not self.FMContextInfo then return end
		SaveDrag = self
		ContextTypes[self:GetAttribute("type")].Drag(self.FMContextInfo)
--		if WOW_PROJECT_ID == WOW_PROJECT_CLASSIC then
		if WOW_PROJECT_ID == WOW_PROJECT_MAINLINE then
			Overlay:RegisterEvent("CURSOR_CHANGED")
		else
			Overlay:RegisterEvent("CURSOR_UPDATE")
		end
		OnEnterLeave(Overlay)
	end

	local function TakeDrop(self, ...)
		if not GetCursorInfo() then return end
		if not SaveDrag or SaveDrag ~= self then
			local settings = NS:GetSettings("Menu").pages
			local column, id, page
			if self.FMContextInfo then
				column = self.FMContextColumn
				id = self.FMContextId
				if SaveDrag and SaveDrag.FMContextId < id then
					id = id - 1
				end
				page = self.FMContextPage
			else
				column = DropZone.Column or 1
				page = ContextMenu.CurrentPage
			end
			if DropZone.Full[column].isfull then return end -- No more items allowed
			local data = { GetCursorInfo() }
			local itemtype = data[1]
			local settingsinsert = { oftype=itemtype, column=column, info=data[ContextTypes[itemtype].CursorID] }
			if SaveDrag then
				tremove(settings[SaveDrag.FMContextPage], SaveDrag.FMContextId)
			end
			if self.FMContextInfo then
				tinsert(settings[page], id, settingsinsert)
			else
				tinsert(settings[page], settingsinsert)
			end
			DropZone:SetPos(column)
		end
		SaveDrag = nil
		ClearCursor()
		OptionsTable.LoadContextMenu()
		OnEnterLeave(Overlay, nil, true)
	end
	OptionsTable.TakeDrop = TakeDrop
	OptionsTable.TakeDrag = TakeDrag

	local function DeletePage(num)
		local page = NS:GetSettings("Menu").pages
		table.wipe(page[num])
		OptionsTable.LoadContextMenu(true)
		DropZone:SetPos(1, true)
	end

	local Settings = NS:GetSettings("Menu")
	Container = CreateFrame("Frame", "$parentMenu", parent)
--	Container:SetSize(110, parent:GetParent().MinMenuHeight - 20)
	Container:SetSize(FrameWidth, FrameHeight)

	Container.Enable = NS.Lib:CreateLedCheckButton("Enabled", Container, Settings, "enabled", "Green")
	Container.Enable:SetSize(20, 20)
	Container.Enable:SetPoint("TOPLEFT", 10, -35)
--	Container.Enable:SetLabel(L.Enabled, "RIGHT")
	NS:AddCheckLabel(Container.Enable, L.Enabled, "RIGHT")
	Container.Enable:SetToolTip(L.GetText(L.EnabledTip, L.ContextMenu))
	CheckButtons[Container.Enable] = true

	Container.Page1 = NS.Lib:CreateLedCheckButton("Page1", Container, Settings, "page1", "Green")
	Container.Page1:SetSize(20, 20)
	Container.Page1:SetPoint("TOPLEFT", Container.Enable, "BOTTOMLEFT", 0, -5)
--	Container.Page1:SetLabel(L.GetText(L.Page, 1), "RIGHT")
	NS:AddCheckLabel(Container.Page1, L.GetText(L.Page, 1), "RIGHT")
	Container.Page1:SetToolTip(L.MenuPage1)
	CheckButtons[Container.Page1] = true
	
	Container.DeleteLabel = NS:CreateLabel(Container, "DeleteLabel", L.Delete)
	Container.DeleteLabel:SetPoint("TOPLEFT", Container.Page1, "BOTTOMLEFT", 30, -10)
	
	local ButtonSettings = { id="DelPage1", nolocale=true, text=L.GetText(L.Page, "1"), tip=L.GetText(L.DeletePage, "1"), icon="DeletePage" }
	Container.DeletePage1 = NS:CreateMenuButton(1, Container, "Button", ButtonSettings, 20, true)
	Container.DeletePage1:SetSize(80, 16)
	Container.DeletePage1:SetPoint("TOPLEFT", Container.DeleteLabel, "BOTTOMLEFT", -30, -10)
	Container.DeletePage1:SetScript("OnClick", function(self)
		DeletePage(1)
	end)

	ButtonSettings = { id="DelPage2", nolocale=true, text=L.GetText(L.Page, "2"), tip=L.GetText(L.DeletePage, "2"), icon="DeletePage" }
	Container.DeletePage2 = NS:CreateMenuButton(2, Container, "Button", ButtonSettings, 20, true)
	Container.DeletePage2:SetSize(80, 16)
	Container.DeletePage2:SetPoint("TOPLEFT", Container.DeletePage1, "BOTTOMLEFT", 0, -10)
	Container.DeletePage2:SetScript("OnClick", function(self)
		DeletePage(2)
	end)
	
	Container.DragHint = NS.Lib:CreateFont("DragHint", Container, L.MenuDragHint)
	Container.DragHint:SetSize(Container.DeletePage2:GetWidth()+20, Container.DeletePage2:GetHeight()*3)
	Container.DragHint:SetPoint("TOPLEFT", Container.DeletePage2, "BOTTOMLEFT", 0, -5)
-- Fizz. ADD Cooldown BG/Text colours

--	self.Menu = NS:GetFrame("Context", "Parent")
	ContextMenu = NS:GetFrame("Context", "Parent")
	Container:SetScript("OnShow", function(self)
		NS:GetOption("ToggleContextmenu")(true, true)
		Overlay:SetScript("OnEnter", OnEnterLeave)
		Overlay:SetScript("OnLeave", function(self, motion)
			OnEnterLeave(self, motion, true)
		end)
		NS:PlaceCopyButton(CopySettings, 10, 0, "TOPRIGHT", Container, "TOPRIGHT", -20, -8)
--		NS:PlaceCopyButton(CopySettings, 0, 38, "TOPLEFT", self, "TOPLEFT", -5, 1)
	end)
	Container:SetScript("OnHide", function(self)
		NS:GetOption("ToggleContextmenu")(true, false)
		Overlay:SetScript("OnEnter", nil)
		Overlay:SetScript("OnLeave", nil)
	end)
	NS:GetOption("ContextFuncs")(OptionsTable)

	ContextTypes = OptionsTable.ContextTypes
--	local menu = self.Menu
--	local menu = Container.Menu
	local scale = UIParent:GetScale()

	Left = (ContextMenu:GetLeft()*scale)+ContextMenu.OffsetX
	Top = (ContextMenu:GetTop()*scale)-ContextMenu.OffsetY
	Width = ((ContextMenu:GetRight()*scale)-Left)/OptionsTable.ColumnCount
	local f = CreateFrame("Frame", "$parentMenuOverlay", Container) -- use a seperate frame so you don't have to mess with the actual menu's scripts etc.
	Overlay = f
	f:SetFrameStrata("MEDIUM")
	f:SetFrameLevel(5)
	f:SetAllPoints(ContextMenu)
	f:SetScript("OnReceiveDrag", function(self, ...)
		TakeDrop(self, ...)
	end)
	f:SetScript("OnEvent", TakeEvent)
	f:SetScript("OnMouseWheel", function(self, ...)
		ContextMenu:GetScript("OnMouseWheel")(ContextMenu, ...)
		self.PageNo:SetText(L.GetText(L.Page, ContextMenu.CurrentPage))
	end)
	f.PageNo = NS.Lib:CreateFont("PageNum", f, L.GetText(L.Page, ContextMenu.CurrentPage), "CENTER", "BOTTOM", nil, 12)	
	f.PageNo:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -20, 20)
	f.PageNo:SetTextColor(1, 1, 0)
	local dz = CreateFrame("Frame", nil, f)
	DropZone = dz
	dz.ColumnWidth = OptionsTable.ColumnWidth
	dz.ItemHeight = OptionsTable.ItemHeight
	dz:SetSize(dz.ColumnWidth, dz.ItemHeight)
	dz.Settings = Settings.pages
--	dz.Menu = menu
	dz.OffsetX = ContextMenu.OffsetX
	dz.OffsetY = ContextMenu.OffsetY
	dz.Texture = dz:CreateTexture()
	dz.Texture:SetAllPoints()
	dz.Texture:SetAlpha(0.5)
	dz.Texture:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	dz.Arrow = dz:CreateTexture()
	dz.Arrow:SetSize(dz:GetHeight(), dz:GetHeight()/2)
	dz.Arrow:SetTexture("Interface\\Buttons\\UI-SortArrow")
	dz.Arrow:SetPoint("CENTER")
	function dz:SetPos(column, init)
		if type(self) == "number" then -- too lazy to fix this atm. Called from button
			init = column
			column = self
			self = DropZone
		end
		self:ClearAllPoints()
		self.Column = nil
		self.Page = nil
		if not column then
			self:SetPoint("TOP", UIParent, "BOTTOM", 0, -50)
		else
			self.Page = ContextMenu.CurrentPage
			self.Column = column
			local settingspage = self.Settings[self.Page]
			for i=1, #self.Full do
				self.Full[i].count = 0
				self.Full[i].isfull = nil
				self.Full[i].text:Hide()
			end
			local items = 0
			for i=1, #settingspage do
				local fullcol = settingspage[i].column
				self.Full[fullcol].count = self.Full[fullcol].count + 1
				if self.Full[fullcol].count >= OptionsTable.ItemsPerCol then
					self.Full[fullcol].isfull = true
					self.Full[fullcol].text:Show()
				end
				if settingspage[i].column == column then
					items = items+1
				end
			end
			if init or items >= OptionsTable.ItemsPerCol then
				self:SetPoint("TOP", UIParent, "BOTTOM", 0, -50)
			else
				self:SetPoint("TOPLEFT", Overlay, ((column-1)*self.ColumnWidth)+self.OffsetX, -((items*self.ItemHeight)+self.OffsetY))
			end
		end
	end
	OptionsTable.SetPos = dz.SetPos
	dz.Full = {} 
	for i=1, OptionsTable.ColumnCount do
		dz.Full[i] = { text=NS.Lib:CreateFont("f"..i, f, upper(L.Full.."!"), "CENTER", "BOTTOM", nil, 14) }
		dz.Full[i].text:SetTextColor(1, 0, 0)
		dz.Full[i].text:SetWidth(OptionsTable.ColumnWidth)
		if i == 1 then
			dz.Full[i].text:SetPoint("BOTTOMLEFT", f, "TOPLEFT")
		else
			dz.Full[i].text:SetPoint("LEFT", dz.Full[i-1].text, "RIGHT")
		end
	end
	dz:SetPos(1, true)
	Container:GetScript("OnShow")(Container)
--	Container:Show()
	return Container
end

NS:RegisterModule("Menu", "Menu", "Menu", ConfigureMenu)
